# -*- coding: utf-8 -*-
# $Id: addportlet.py 78 2008-09-24 15:39:05Z gael.le-mignot $

from zope.interface import implements, Interface, implementsOnly
from zope.component import adapts, getMultiAdapter, queryUtility
from Acquisition import aq_inner, aq_base, aq_parent

from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import utils

from Products.CMFPlone import PloneMessageFactory as _
from plone.app.portlets.portlets import base

from zope import schema
from zope.formlib import form

from ps import nozcml

from plone.app.portlets.portlets import navigation 
from plone.app.form.widgets.uberselectionwidget import UberSelectionWidget
from plone.memoize.instance import memoize

from plone.app.layout.navigation.navtree import buildFolderTree
from plone.app.layout.navigation.interfaces import INavtreeStrategy
from plone.app.layout.navigation.interfaces import INavigationQueryBuilder

class IMultiLevelNavigationPortlet(navigation.INavigationPortlet):
    """A portlet for a multilevel navigation"""

    depth = schema.Int(
            title=_(u"label_navigation_depth",
                    default=u"Number of levels to display"),
            description=_(u"help_navigation_depth",
                default=u"This will indicate how many levels to display below the current node. Not to be confused with the previous option, which indicates until when the portlet is displayed at all."),
            default=1,
            required=False)
    
class Assignment(navigation.Assignment):
    implementsOnly(IMultiLevelNavigationPortlet)

    depth=1
    
    def __init__(self, depth=1, **kwargs):
        super(Assignment, self).__init__(**kwargs)
        self.depth = depth

class MultilevelQueryBuilder(navigation.QueryBuilder):
    """
    This is a custom query builder, but it requires the request too...
    """
    def __init__(self, context, portlet, request):
        super(MultilevelQueryBuilder, self).__init__(context, portlet)

        # Fix the query to handle our own stuff
        objPhysicalPath = context.getPhysicalPath()
        if utils.isDefaultPage(context, request):
            objPhysicalPath = objPhysicalPath[:-1]
        objPath = '/'.join(objPhysicalPath)
        self.query['path'] = { 'query': objPath,
                               'multilevel_navtree': portlet.depth or 1 }

class Renderer(navigation.Renderer):
    @memoize
    def getNavTree(self, _marker=[]):
        context = aq_inner(self.context)
        
        # Special case - if the root is supposed to be pruned, we need to
        # abort here

        queryBuilder = MultilevelQueryBuilder(context, self.data, self.request)
        query = queryBuilder()
        strategy = getMultiAdapter((context, self.data), INavtreeStrategy)

        return buildFolderTree(context, obj=context, query=query, strategy=strategy)

class AddForm(base.AddForm):
    form_fields = form.Fields(IMultiLevelNavigationPortlet)
    form_fields['root'].custom_widget = UberSelectionWidget
    label = _(u"Add Multilevel Navigation Portlet")
    description = _(u"This portlet display a navigation tree.")

    def create(self, data):
        return Assignment(name=data.get('name', u""),
                          root=data.get('root', u""),
                          currentFolderOnly=data.get('currentFolderOnly', False),
                          includeTop=data.get('includeTop', False),
                          topLevel=data.get('topLevel', 0),
                          bottomLevel=data.get('bottomLevel', 0),
                          depth=data.get('depth', 1),
                          )

class EditForm(base.EditForm):
    form_fields = form.Fields(IMultiLevelNavigationPortlet)
    form_fields['root'].custom_widget = UberSelectionWidget
    label = _(u"Edit Multilevel Navigation Portlet")
    description = _(u"This portlet display a navigation tree.")


